module FaaStRuby
  module Command
    module Account
      require 'faastruby/user'
      require 'faastruby/cli/new_credentials'

      PASSWORD_REGEX = /^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[@$!%*?&])[A-Za-z\d@$!%*?&]{8,50}$/
      EMAIL_REGEX = /\A[a-z0-9\-\.+]+@[a-z\d\-]+(\.[a-z]+)*\.[a-z]+\z/i

      class AccountBaseCommand < BaseCommand
        def password_is_confirmed?(password, password_confirmation)
          return false if password.nil?

          password == password_confirmation
        end

        def password_is_valid?(password)
          password.match(PASSWORD_REGEX)
        end

        def prompt(command:, label:, secure: true)
          puts ""
          puts "#{command}"
          print label

          if secure
            STDIN.noecho(&:gets).chomp
          else
            STDIN.gets.chomp
          end
        end

        def prompt_for_password
          password = prompt(command: "\nNow type in a password. It must contain 8 to 50 characters and have at least one uppercase letter, one lowercase letter, one number and one special character @ $ ! % * ? &",
                            label: "Password: ")

          until password_is_valid?(password) do
            password = prompt(command: "\nYour password must contain 8 to 50 characters and have at least one uppercase letter, one lowercase letter, one number and one special character @ $ ! % * ? &\nPlease try again.".red,
                              label: "Password: ")
          end

          password
        end

        def prompt_to_confirm_password(password)
          password_confirmation = prompt(command: "\nNow type that password again",
                                         label: "Confirm password: ")
            
          until password_is_confirmed?(password, password_confirmation)
            print "\n\nHmm... those passwords did not match. Let's try again".red

            password = prompt_for_password
            password_confirmation = prompt(command: "\nNow type that password again",
                                           label: "Confirm password: ")
          end
        end
      end
    end
  end
end